// via Dan Fekete
// https://www.youtube.com/watch?v=FdTGeYKB5NM

// Ring Pin reference circle diameter
D = 42;
// Number of ring pins
N = 10;
// Pin diameter
d_p = 5;
// eccentricity
//e = 0.3*d_p;
e = 1;

// Number of rolling pins
n = 4;
// Roller Pin diameter
d_r = 3;
// Roller Pin bore
d_rb = d_r + 2*e;
// Roller Pin reference circle diameter
d_d = 22;

// gear thickness
th = 5;

$fn = ($preview) ? 12 : 64;
i = N - 1; // transmission ratio


for(m=[0,1]) mirror([0,0,m]) {
 translate([cos(360*$t)*e,sin(360*$t)*e,th])
  cycloid_gear();
 color("silver")
  translate([cos(180+360*$t)*e,sin(180+360*$t)*e,])
   mirror([1,0,0]) cycloid_gear();
}
 
radial_pins(n=n,ref_d=d_d,d=d_r,th=5*th,center=true);  // roller pins
rotate([0,0,360*$t/(N)])
 radial_pins(n=N,ref_d=D,d=d_p,th=4*th,center=true);  // ring pins

module cycloid_gear() {
 render() difference() {
  cycloid(N=i, base_circ_r=(D*i/N)/2, pin_d=d_p, e=e, h=th);
  translate([0,0,-.5])
   radial_pins(n=n,ref_d=d_d,d=d_rb,th=th+1); // roller pin bores
 }
}

module radial_pins(n,ref_d,d,th,center) {
 for(a=[0:360/n:359]) let(x=cos(a)*ref_d/2,y=sin(a)*ref_d/2) {
  translate([x,y])
   cylinder(d=d,h=th,center=center);
 }
}

module bearing_cut(d_ball=4.42,r_ref,w) {
 rotate_extrude($fn=4*$fn)
  translate([r_ref,0,0])
   circle($fn=$fn/4,d=d_ball);
 difference() {
  cylinder(r=r_ref+d_ball*1/3,h=w+0.01,center=true);
  cylinder(r=r_ref-d_ball*1/3,h=w+1,center=true);
 }
 translate([r_ref,0])
  cylinder(d1=.98*d_ball,d2=1.1*d_ball,h=w/2+0.01);
}

module cycloid(N, base_circ_r, pin_d, e, h=1) {
 $fn = $fn ? $fn : 12;
 roling_r = (2*PI*base_circ_r) / ((N+1)*2*PI);
 points = [ for(a=[0:360/($fn*N/2):359])
  [ (base_circ_r + roling_r) * cos(a),
    (base_circ_r + roling_r) * sin(a)]
  +
  [ e * cos(a * (2*PI*base_circ_r) / (2*PI*roling_r)),
    e * sin(a * (2*PI*base_circ_r) / (2*PI*roling_r))]
 ];
 rotate([0,0,180/N])
  linear_extrude(h)
   offset(r=-pin_d/2)
    polygon(points);
}